﻿using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Common;
using MvcDesign.Core.CoreServices;
using MvcDesign.Core.DataAccess;

namespace MvcDesign.Forum.ForumServices
{
    class ForumViewService : IForumViewService
    {
        private readonly IUserViewService _userViewService;
        private readonly IForumPathViewService _pathViewService;

        public ForumViewService(IUserViewService userViewService, IForumPathViewService pathViewService)
        {
            _userViewService = userViewService;
            _pathViewService = pathViewService;
        }

        public List<ForumViewModel.Section> GetAllSections()
        {
            using (var domainModel = new DataContext())
            {
                return domainModel.GetCollection<ForumDataModel.Section>().Where(x => !x.ParentSectionId.HasValue).ToList().Select(s => GetSection(domainModel, s)).ToList();
            }
        }

        public ForumViewModel.Section GetSection(long sectionId)
        {
            using (var domainModel = new DataContext())
            {
                var section = domainModel.GetCollection<ForumDataModel.Section>().Find(sectionId);
                var model = GetSection(domainModel, section);

                if (section.IsForum)
                {
                    foreach (var thread in domainModel.GetCollection<ForumDataModel.Thread>().Where(t => t.SectionId == section.Id))
                    {
                        if (!thread.Id.HasValue)
                        {
                            continue;
                        }

                        model.Threads.Add(GetSectionThread(thread.Id.Value));
                    }
                }

                return model;
            }
        }

        private ForumViewModel.Section GetSection(DataContext domainModel, ForumDataModel.Section section)
        {
            if (!section.Id.HasValue)
            {
                return null;
            }

            var requestContext = ((MvcHandler) HttpContext.Current.Handler).RequestContext;

            var model = new ForumViewModel.Section
            {
                Name = section.Name,
                Id = section.Id,
                Url = new UrlHelper(requestContext).RouteUrl(new { application = "Forum", action = "Section", id = section.Id.Value }),
                Description = section.Description,
                Forums = domainModel.GetCollection<ForumDataModel.Section>().Where(s => s.ParentSectionId == section.Id).ToList().Select(s => GetSectionForum(domainModel, s)).ToList(),
                Threads = new List<ForumViewModel.SectionThread>(),
                Path = _pathViewService.GetFullPathSection(section.Id.Value, true)
            };

            return model;
        }

        private static void GetChildForums(DataContext domainModel, long forumId, ICollection<long> ids)
        {
            foreach (var subForumId in domainModel.GetCollection<ForumDataModel.Section>().Where(s => s.ParentSectionId == forumId).Select(s => s.Id))
            {
                ids.Add(subForumId.Value);
                GetChildForums(domainModel, subForumId.Value, ids);
            }
        }

        private ForumViewModel.SectionForum GetSectionForum(DataContext domainModel, ForumDataModel.Section forum)
        {
            if (!forum.Id.HasValue)
            {
                return null;
            }

            var model = new ForumViewModel.SectionForum
            {
                Name = forum.Name,
                Description = forum.Description,
                Url = MvcHelper.UrlHelper.RouteUrl(new { application = "Forum", action = "Section", id = forum.Id.Value }),
                HasNewPosts = false, // TBD
                Id = forum.Id.Value
            };

            var forumIds = new List<long> { forum.Id.Value };
            GetChildForums(domainModel, forum.Id.Value, forumIds);

            var threadIds = domainModel.GetCollection<ForumDataModel.Thread>().Where(t => forumIds.Contains(t.SectionId)).Select(t => t.Id);

            model.Threads = threadIds.Count();
            model.Posts = domainModel.GetCollection<ForumDataModel.Post>().Count(p => threadIds.Contains(p.ThreadId));

            var lastPost = domainModel.GetCollection<ForumDataModel.Post>().Where(p => threadIds.Contains(p.ThreadId)).OrderByDescending(p => p.Created).FirstOrDefault();

            if (lastPost != null)
            {
                model.LastAuthor = _userViewService.GetUser(lastPost.AuthorId);
                model.LastPostTime = lastPost.Created;
                model.LastPostTitle = lastPost.Title;
                if (string.IsNullOrEmpty(model.LastPostTitle))
                {
                    model.LastPostTitle = domainModel.GetCollection<ForumDataModel.Thread>().Find(lastPost.ThreadId).Name;
                }
                model.LastPostUrl = MvcHelper.UrlHelper.RouteUrl(new { application = "Forum", action = "Thread", id = lastPost.ThreadId, page = "last" });
            }

            model.Moderators = new List<Core.CoreViewModel.User>();

            return model;
        }

        public ForumViewModel.SectionThread GetSectionThread(long threadId)
        {
            using (var domainModel = new DataContext())
            {
                var thread = domainModel.GetCollection<ForumDataModel.Thread>().Find(threadId);
                return GetSectionThread(domainModel, thread);
            }
        }

        private const int PostsPerPage = 20;

        public ForumViewModel.Thread GetThread(long threadId, int pageNumber)
        {
            using (var domainModel = new DataContext())
            {
                var thread = domainModel.GetCollection<ForumDataModel.Thread>().Find(threadId);

                if (!thread.Id.HasValue)
                {
                    return null;
                }

                var posts = domainModel.GetCollection<ForumDataModel.Post>().Where(p => p.ThreadId == threadId).OrderBy(p => p.Created);

                var postsCount = posts.Count();
                var pagesCount = (postsCount + PostsPerPage - 1) / PostsPerPage;

                var model = new ForumViewModel.Thread
                {
                    Id = thread.Id.Value,
                    Name = thread.Name,
                    Description = thread.Description,
                    Page = pageNumber,
                    Pages = pagesCount,
                    Posts = posts.Skip((pageNumber - 1) / PostsPerPage).Take(PostsPerPage).ToList().Select(p => GetPost(p.Id ?? 0)).ToList(),
                    Path = _pathViewService.GetFullPathThread(thread.Id.Value)
                };

                return model;
            }
        }

        public int GetThreadPostCount(long threadId)
        {
            using (var domainModel = new DataContext())
            {
                return domainModel.GetCollection<ForumDataModel.Post>().Where(p => p.ThreadId == threadId).OrderBy(p => p.Created).Count();
            }
        }

        public int GetThreadLastPage(long threadId)
        {
            var postsCount = GetThreadPostCount(threadId);
            var pagesCount = (postsCount + PostsPerPage - 1) / PostsPerPage;
            return pagesCount;
        }

        private ForumViewModel.SectionThread GetSectionThread(DataContext domainModel, ForumDataModel.Thread thread)
        {
            if (!thread.Id.HasValue)
            {
                return null;
            }

            var posts = domainModel.GetCollection<ForumDataModel.Post>().Where(p => p.ThreadId == thread.Id);

            var model = new ForumViewModel.SectionThread
            {
                Id = thread.Id.Value,
                Name = thread.Name,
                Description = thread.Description,
                Url = MvcHelper.UrlHelper.RouteUrl(new
                {
                    application = Constants.Applications.Forum.Name,
                    action = Constants.Applications.Forum.Actions.Thread,
                    id = thread.Id
                }),
                Posts = posts.Count(),
                Created = thread.Created,
                CreatedBy = _userViewService.GetUser(thread.AuthorId)
            };

            if (model.Posts > 0)
            {
                var lastPost = posts.OrderByDescending(p => p.Created).First();
                model.LastPostTime = lastPost.Created;
                model.LastPostTitle = lastPost.Title;
                model.LastAuthor = _userViewService.GetUser(lastPost.AuthorId);
                model.LastPostUrl = model.Url + "#" + lastPost.Id;
            }

            return model;
        }

        public ForumViewModel.Post GetPost(long postId)
        {
            using (var domainModel = new DataContext())
            {
                var post = domainModel.GetCollection<ForumDataModel.Post>().Find(postId);
                var model = new ForumViewModel.Post
                {
                    Id = postId,
                    Title = post.Title,
                    Text = post.Text,
                    Created = post.Created,
                    Author = _userViewService.GetUser(post.AuthorId),
                    Modified = post.Modified,
                    ModifiedBy = post.ModifiedById.HasValue ? _userViewService.GetUser(post.ModifiedById.Value) : null
                };

                return model;
            }
        }
    }
}
